package main

import (
    "flag"
    "io"
    "log"
    "fmt"
    "net/http"
    "os"
    "strings"
)

const usage = `Usage: https_server

Serve a sample page over HTTPs.

positional arguments
  [HOST]:PORT

  A "host:port" address taht the target listson on for HTTPS requests.

options:
  -cert TLS_CERT
    The server's TLS certificate file.

    Default: server.crt

  -key TLS_PRIVATE_KEY
    The server's TLS private key.
   
    Default: server.key

  -help
    Display this usage statement and exit

example:
$ ./https_server -cert my.cert -key my.key 127.0.0.1:12345
`

type Options struct {
    certFile string
    keyFile string
    hostport string
}

func die(format string, a ...any) {
    if !strings.HasSuffix(format, "\n") {
        format += "\n"
    }
    fmt.Fprintf(os.Stderr, format, a...)
    os.Exit(1)
}

func printUsage() {
    fmt.Fprintf(os.Stderr, "%s", usage)
}

func parseOptions() *Options {
    options := Options{}

    flag.Usage = printUsage
    flag.StringVar(&options.certFile, "cert", "server.crt", "")
    flag.StringVar(&options.keyFile, "key", "server.key", "")

    flag.Parse()

    if flag.NArg() != 1 {
        die("expected one positional argument but got %d", flag.NArg())
    }

    options.hostport = flag.Arg(0)

    return &options
}

func main() {
    options := parseOptions()

    http.HandleFunc("/", func(w http.ResponseWriter, req *http.Request) {
        io.WriteString(w, "Hello, TLS!\n")
    })

    log.Printf("About to listen on %s.  Go to https://%s/",
            options.hostport, options.hostport)

    err := http.ListenAndServeTLS(options.hostport, options.certFile, options.keyFile, nil)
    log.Fatal(err)
}
